#include "connection.h"

using namespace std;

#define MESSAGE_SIZE 512

connection::connection(string hostname, int port) {
    // default value 
    sock = -1;

    // int to string
    stringstream service;
    service << port;

    // address initialization
    memset(&addr, 0, sizeof addr);

    // both IPv4 and IPv6 are supported
    addr.ai_family = PF_UNSPEC;
    addr.ai_socktype = SOCK_STREAM;

    int err = getaddrinfo(hostname.c_str(), service.str().c_str(), &addr, &ai);

    if (err != 0) {
        throw runtime_error("could not obtain the hostname: " + *gai_strerror(err));
    }

    // socket initialization
    sock = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);

    if (sock == -1) {
        throw runtime_error("unable to create socket");
    }

    // and finally, establishing the connection
    if (connect(sock, ai->ai_addr, ai->ai_addrlen) == -1) {
        throw runtime_error("unable to connect");
    }

    freeaddrinfo(ai);

#ifdef DEBUG
    cout << "Connected to " << hostname << "." << endl;
#endif
}

string connection::read() {
    char s[MESSAGE_SIZE];
    int len;

#ifdef DEBUG
    cout << "Reading..." << endl;
#endif

    len = recv(sock, s, MESSAGE_SIZE, 0);

    if (len == -1) {
        throw runtime_error("server read error");
    }

    s[len] = '\0';
   
    return s;
}

void connection::write(string message) {
#ifdef DEBUG
    string m = message;

    cout << "Writing \"" << m.erase(m.find_last_not_of("\n\r\t") + 1)
         << "\"..." << endl;
#endif

    if (send(sock, message.c_str(), message.size(), 0) == -1) {
        throw runtime_error("server write error");
    }
}

connection::~connection() {
    if (sock != -1) {
        close(sock);
    }

#ifdef DEBUG
    cout << "Disconnected." << endl;
#endif
}
