#include "pop3.h"

using namespace std;

#define PORT_DEFAULT 110
#define ID_SIZE 40
#define CRLF "\r\n"

pop3::pop3(string hostname, string username, string password, int port) {
    if (port < 1 || port > 65535) {
        port = PORT_DEFAULT;
    }

#ifdef DEBUG
    cout << "Hostname: " << hostname << endl
         << "Username: " << username << endl
         << "Password: " << password << endl
         << "Port: " << port << endl;
#endif

    conn = new connection(hostname, port);

    string greeting = conn->read();
    
    if (!is_ok(greeting)) {
        throw runtime_error("server have not sent a welcome message");
    }

#ifdef DEBUG
    cout << "Welcome message: " << greeting.substr(4, string::npos);
#endif

    // authentization
    if (!is_ok(communicate("USER " + username, false))) {
        throw runtime_error("wrong username");
    }

    if (!is_ok(communicate("PASS " + password, false))) {
        throw runtime_error("wrong password");
    }
}

bool pop3::is_ok(string message) {
    return message.substr(0, 3) == "+OK";
}

string pop3::communicate(string message, bool multiline) {
    conn->write(message + CRLF);

    string r, response = "";

    // an initial line
    r = conn->read();

    if (is_ok(r) && !multiline) {
        return r;
    }
    else if (r.substr(0, 4) == "-ERR" && r.size() > 5) {
        r = r.substr(5, string::npos);
        r.erase(r.find_last_not_of("\n\r\t") + 1);

        return r;
    }
    else if (!multiline) {
        throw runtime_error("unexpected server response");
    }

    // multiline response does not need the initial line
    r = r.substr(r.find(CRLF) + 2, string::npos);

    // process rest of the multiline response
    while (r.size() > 5 &&
           r.substr(r.size() - 5, string::npos) != (CRLF "." CRLF)) {
        response += r;
        r = conn->read();
    }

    if (r.size() > 5) {
        response += r.substr(0, r.size() - 5);
    }

    return response;
}

string pop3::list() {
    return communicate("LIST", true);
}

string pop3::retr(string id_raw) {
    string::iterator i;
    string id;

    // only ASCII characters from the range 0x21-0x7E are allowed
    for (i = id_raw.begin(); i < id_raw.end(); i++) {
        if (*i >= 0x21 && *i <= 0x7E) {
            id += *i;
        }
    }

#ifdef DEBUG
    cout  << "Message ID: " << id << endl;
#endif

    if (id.empty() || id.size() > ID_SIZE) {
        throw runtime_error("Illegal message identifier. " \
                            "Please use reasonably long identifiers that " \
                            "contain alphanumerical characters only.");
    }

    return communicate("RETR " + id, true);
}

pop3::~pop3() {
    communicate("QUIT", false);

    delete conn;
}
